@@ -29,9 +29,6 @@
 #'       patterns.
 #'   * `"gap"` allows transitions between non-adjacent states, with
 #'       transitions weighted by the gap size.
-#'   * `"window"` creates transitions between all states within a
-#'       sliding window, capturing local relationships
-#'       (several sequences together).
 #'   * `"reverse"` considers the sequences in reverse order
 #'       (resulting in what is called a reply network in some contexts).
 #'       The resulting weight matrix is the transpose of the `"frequency"`
@@ -57,52 +54,56 @@
 #'       `ties.method = "average"`.
 #'
 #' @param cols An `expression` giving a tidy selection of columns that should
-#'   be considered as sequence data. By default, all columns are used. Ignored
-#'   for `matrix`, `tna_data` and `tsn` type `x`.
+#'   be considered as sequence data. By default, all columns are used.
 #' @param params A `list` of additional arguments for models of specific
 #'   `type`. The potential elements of this list are:
 #'
 #'   * `n_gram`: An `integer` for n-gram transitions specifying the number of
-#'       adjacent events. The default value is 2.
+#'     adjacent events. The default value is 2.
 #'   * `max_gap`: An `integer` for the gap-allowed transitions specifying the
-#'       largest allowed gap size. The default is 1.
+#'     largest allowed gap size. The default is 1.
+#'   * `windowed`: Perform the model estimation by window. Supported for
+#'     `relative`, `frequency` and `co-occurrence` `type`s.
 #'   * `window_size`: An `integer` for the sliding window transitions
-#'       specifying the window size. The default is 2.
+#'     specifying the window size. The default is 2.
+#'   * `window_type`: A `character` string that defines the window type.
+#'     Either `"rolling"` or `"tumbling"`.
 #'   * `weighted`: A `logical` value. If `TRUE`, the transitions
-#'      are weighted by the inverse of the sequence length. Can be used for
-#'      frequency, co-occurrence and reverse model types. The default is
+#'     are weighted by the inverse of the sequence length. Can be used for
+#'     frequency, co-occurrence and reverse model types. The default is
 #'      `FALSE`.
 #'   * `direction`: A `character` string specifying the direction of attention
-#'      for models of `type = "attention"`. The available options are
-#'      `"backward"`, `"forward"`, and `"both"`, for backward attention,
-#'      forward attention, and bidirectional attention, respectively.
-#'      The default is `"forward"`.
+#'     for models of `type = "attention"`. The available options are
+#'     `"backward"`, `"forward"`, and `"both"`, for backward attention,
+#'     forward attention, and bidirectional attention, respectively.
+#'     The default is `"forward"`.
 #'   * `decay`: A `function` that specifies the decay of the weights between
-#'      two time points at a specific distance. The function should take three
-#'      arguments: `i`, `j` and `lambda`, where `i` and `j` are `numeric`
-#'      vectors of time values, and `lambda` is  a `numeric` value for the
-#'      decay rate. The function should return a `numeric` vector of weights.
-#'      The default is `function(i, j, lambda) exp(-abs(i - j) / lambda)`.
+#'     two time points at a specific distance. The function should take three
+#'     arguments: `i`, `j` and `lambda`, where `i` and `j` are `numeric`
+#'     vectors of time values, and `lambda` is  a `numeric` value for the
+#'     decay rate. The function should return a `numeric` vector of weights.
+#'     The default is `function(i, j, lambda) exp(-abs(i - j) / lambda)`.
 #'   * `lambda`: A `numeric` value for the decay rate. The default is 1.
 #'   * `time`: A `matrix` or a `data.frame` providing the time values
-#'      for each sequence and at time index. For `tna_data` objects, this can
-#'      also be a logical value, where `TRUE` will use the `time_data` element
-#'      of `x` for the time values. `Date` values are converted to `numeric`.
+#'     for each sequence and at time index. For `tna_data` objects, this can
+#'     also be a logical value, where `TRUE` will use the `time_data` element
+#'     of `x` for the time values. `Date` values are converted to `numeric`.
 #'   * `duration`: A `matrix` or a `data.frame` providing the
-#'      time spent in each state for each sequence and time index.
-#'      This is an alternative to `time`.
+#'     time spent in each state for each sequence and time index.
+#'     This is an alternative to `time`.
 #'
+#' @param concat An `integer` for the number of consecutive sequences
+#'   to concatenate. The default is 1 (no concatenation).
 #' @param inits An optional `numeric` vector of initial state probabilities
 #'   for each state. The vector will be scaled to unity.
-#'   Ignored if `x` is not a `matrix`.
 #' @param begin_state A `character` string for an additional begin state.
 #'   This state is added as the first observation for every sequence to
 #'   signify the beginning of the sequence
 #' @param end_state A `character` string for an additional end state.
 #'   This state is added as the last observation for every sequence to
-#'   siginify the end of the sequence.
+#'   signify the end of the sequence.
 #' @param ... Ignored. For the `build_model` aliases (e.g., `tna`), this
-#' argument matches the actual arguments to `build_model` beside `x`.
+#'   argument matches the actual arguments to `build_model` beside `x`.
 #' @return An object of class `tna` which is a `list` containing the
 #'   following elements:
 #'
@@ -120,19 +121,16 @@
 #' model <- build_model(group_regulation)
 #' print(model)
 #'
-build_model <- function(x, type = "relative", scaling = character(0L),
-                        cols = tidyselect::everything(), params = list(),
-                        inits, begin_state, end_state) {
+build_model <- function(x, ...) {
   UseMethod("build_model")
 }
 
 #' @export
 #' @rdname build_model
 build_model.default <- function(x, type = "relative", scaling = character(0L),
-                                cols = tidyselect::everything(),
-                                params = list(), inits,
-                                begin_state, end_state) {
+                                params = list(), inits, ...) {
   check_missing(x)
+  check_dots(...)
   x <- try_(as.matrix(x))
   stopifnot_(
     !inherits(x, "try-error"),
@@ -150,9 +148,9 @@ build_model.default <- function(x, type = "relative", scaling = character(0L),
 #' @export
 #' @rdname build_model
 build_model.matrix <- function(x, type = "relative", scaling = character(0L),
-                               cols = tidyselect::everything(), params = list(),
-                               inits, begin_state, end_state) {
+                               params = list(), inits, ...) {
   check_missing(x)
+  check_dots(...)
   x <- try_(data.matrix(x))
   stopifnot_(
     !inherits(x, "try-error"),
@@ -216,9 +214,10 @@ build_model.matrix <- function(x, type = "relative", scaling = character(0L),
 
 build_model.stslist <- function(x, type = "relative", scaling = character(0L),
                                 cols = tidyselect::everything(),
-                                params = list(), inits,
-                                begin_state, end_state) {
+                                params = list(), concat = 1L,
+                                begin_state, end_state, ...) {
   check_missing(x)
+  check_dots(...)
   check_class(x, "stslist")
   type <- check_model_type(type)
   scaling <- check_model_scaling(scaling)
@@ -226,6 +225,7 @@ build_model.stslist <- function(x, type = "relative", scaling = character(0L),
   x <- create_seqdata(
     x = x,
     cols = cols,
+    concat = concat,
     begin_state = begin_state,
     end_state = end_state
   )
@@ -246,16 +246,22 @@ build_model.stslist <- function(x, type = "relative", scaling = character(0L),
 build_model.data.frame <- function(x, type = "relative",
                                    scaling = character(0L),
                                    cols = tidyselect::everything(),
-                                   params = list(), inits,
-                                   begin_state, end_state) {
+                                   concat = 1L, params = list(),
+                                   begin_state, end_state, ...) {
   check_missing(x)
   check_class(x, "data.frame")
+  check_dots(...)
   type <- check_model_type(type)
   scaling <- check_model_scaling(scaling)
   cols <- get_cols(rlang::enquo(cols), x)
+  params$window_span <- attr(x, "window_span")
+  params$window_size <- attr(x, "window_size")
+  params$windowed <- !is.null(params$window_size) ||
+    !is.null(params$window_span)
   x <- create_seqdata(
     x = x,
     cols = cols,
+    concat = concat,
     begin_state = begin_state,
     end_state = end_state
   )
@@ -274,11 +280,11 @@ build_model.data.frame <- function(x, type = "relative",
 #' @export
 #' @rdname build_model
 build_model.tna_data <- function(x, type = "relative", scaling = character(0L),
-                                 cols = tidyselect::everything(),
-                                 params = list(), inits, begin_state,
-                                 end_state) {
+                                 params = list(), concat = 1L,
+                                 begin_state, end_state, ...) {
   check_missing(x)
   check_class(x, "tna_data")
+  check_dots(...)
   type <- check_model_type(type)
   scaling <- check_model_scaling(scaling)
   wide <- cbind(x$sequence_data, x$meta_data)
@@ -299,6 +305,7 @@ build_model.tna_data <- function(x, type = "relative", scaling = character(0L),
   x <- create_seqdata(
     x = wide,
     cols = names(x$sequence_data),
+    concat = concat,
     begin_state = begin_state,
     end_state = end_state
   )
@@ -317,29 +324,28 @@ build_model.tna_data <- function(x, type = "relative", scaling = character(0L),
 #' @export
 #' @rdname build_model
 build_model.tsn <- function(x, type = "relative", scaling = character(0L),
-                            cols = tidyselect::everything(), params = list(),
-                            inits, begin_state, end_state) {
+                            params = list(), concat = 1L,
+                            begin_state, end_state, ...) {
   check_missing(x)
   check_class(x, "tsn")
+  check_dots(...)
   type <- check_model_type(type)
   scaling <- check_model_scaling(scaling)
-  id <- attr(x, "id_col")
-  state <- attr(x, "state_col")
-  time <- attr(x, "time_col")
   wide <- x |>
     dplyr::select(
-      c(!!rlang::sym(id), !!rlang::sym(state), !!rlang::sym(time))
+      c(!!rlang::sym("id"), !!rlang::sym("state"), !!rlang::sym("time"))
     ) |>
     tidyr::pivot_wider(
-      id_cols = attr(x, "id_col"),
-      values_from = !!rlang::sym(state),
-      names_from = !!rlang::sym(time),
+      id_cols = "id",
+      values_from = !!rlang::sym("state"),
+      names_from = !!rlang::sym("time"),
       names_prefix = "T"
     ) |>
-    dplyr::select(!(!!rlang::sym(id)))
+    dplyr::select(!(!!rlang::sym("id")))
   x <- create_seqdata(
     x = wide,
     cols = names(wide),
+    concat = concat,
     begin_state = begin_state,
     end_state = end_state
   )
@@ -355,44 +361,45 @@ build_model.tsn <- function(x, type = "relative", scaling = character(0L),
   )
 }
 
-#' @export
-#' @rdname build_model
-build_model.tsn_ews <- function(x, type = "relative", scaling = character(0L),
-                                cols = tidyselect::everything(),
-                                params = list(), inits,
-                                begin_state, end_state) { # nocov start
-  check_missing(x)
-  check_class(x, "tsn_ews")
-  type <- check_model_type(type)
-  scaling <- check_model_scaling(scaling)
-  id <- attr(x, "id_col")
-  cls <- attr(x, "classification")
-  wide <- cls |>
-    dplyr::select(
-      c(!!rlang::sym("state"), !!rlang::sym("time"))
-    ) |>
-    tidyr::pivot_wider(
-      values_from = !!rlang::sym("state"),
-      names_from = !!rlang::sym("time"),
-      names_prefix = "T"
-    )
-  x <- create_seqdata(
-    x = wide,
-    cols = seq_len(ncol(wide)),
-    begin_state = begin_state,
-    end_state = end_state
-  )
-  model <- initialize_model(x, type, scaling, params)
-  build_model_(
-    weights = model$weights,
-    inits = model$inits,
-    labels = model$labels,
-    type = type,
-    scaling = scaling,
-    data = x,
-    params = params
-  )
-} # nocov end
+#' #' @export
+#' #' @rdname build_model
+#' build_model.tsn_ews <- function(x, type = "relative", scaling = character(0L),
+#'                                 params = list(), concat = 1L,
+#'                                 begin_state, end_state, ...) { # nocov start
+#'   check_missing(x)
+#'   check_class(x, "tsn_ews")
+#'   check_dots(...)
+#'   type <- check_model_type(type)
+#'   scaling <- check_model_scaling(scaling)
+#'   id <- attr(x, "id_col")
+#'   cls <- attr(x, "classification")
+#'   wide <- cls |>
+#'     dplyr::select(
+#'       c(!!rlang::sym("state"), !!rlang::sym("time"))
+#'     ) |>
+#'     tidyr::pivot_wider(
+#'       values_from = !!rlang::sym("state"),
+#'       names_from = !!rlang::sym("time"),
+#'       names_prefix = "T"
+#'     )
+#'   x <- create_seqdata(
+#'     x = wide,
+#'     cols = seq_len(ncol(wide)),
+#'     concat = concat,
+#'     begin_state = begin_state,
+#'     end_state = end_state
+#'   )
+#'   model <- initialize_model(x, type, scaling, params)
+#'   build_model_(
+#'     weights = model$weights,
+#'     inits = model$inits,
+#'     labels = model$labels,
+#'     type = type,
+#'     scaling = scaling,
+#'     data = x,
+#'     params = params
+#'   )
+#' } # nocov end
 
 # Aliases -----------------------------------------------------------------
 
@@ -535,13 +542,12 @@ build_model_ <- function(weights, inits = NULL, labels = NULL,
 #' @param x A `data.frame` or a `stslist` object.
 #' @param cols An `character` vector of column names.
 #' @param alphabet Optional `character` vector of the alphabet.
+#' @param concat How many consecutive sequences should be concatenated?
 #' @param begin_state Optional `character` string giving the begin state.
 #' @param end_state Optional `character` string giving the end state.
 #' @noRd
-create_seqdata <- function(x, cols, alphabet, begin_state, end_state) {
-  # if (is.numeric(cols)) {
-  #   stop("Numeric cols detected")
-  # }
+create_seqdata <- function(x, cols, alphabet, concat = 1L,
+                           begin_state, end_state) {
   cols <- which(names(x) %in% cols)
   if (inherits(x, "stslist")) {
     alphabet <- attr(x, "alphabet")
@@ -574,6 +580,26 @@ create_seqdata <- function(x, cols, alphabet, begin_state, end_state) {
       )
     )
   )
+  if (concat > 1L) {
+    n <- nrow(x)
+    k <- ncol(x)
+    m <- n %/% concat
+    modulus <- n %% concat
+    extra <- modulus > 0L
+    x_cc <- matrix(NA_integer_, nrow = m + extra, ncol = concat * ncol(x))
+    for (i in seq_len(m)) {
+      idx <- seq((i - 1L) * concat + 1L, i * concat)
+      x_cc[i, ] <- c(t(x[idx, , drop = FALSE]))
+    }
+    if (extra) {
+      idx <- seq(1L, k)
+      for (i in seq_len(modulus)) {
+        x_cc[m + 1L, idx] <- x[m * concat + i, , drop = FALSE]
+        idx <- idx + k
+      }
+    }
+    x <- x_cc
+  }
   if (!missing(begin_state)) {
     x <- cbind(1L, x + 1L)
     alphabet <- c(begin_state, alphabet)
@@ -638,6 +664,9 @@ initialize_model <- function(x, type, scaling, params, transitions = FALSE) {
 #' @param params Parameters for the transition model.
 #' @noRd
 compute_transitions <- function(m, a, type, params) {
+  if (isTRUE(params$windowed)) {
+    return(compute_transitions_windowed(m, a, type, params))
+  }
   n <- nrow(m)
   p <- ncol(m)
   idx <- seq_len(n)
@@ -697,19 +726,6 @@ compute_transitions <- function(m, a, type, params) {
         trans[new_trans] <- trans[new_trans] + 1.0 / (j - i)
       }
     }
-  } else if (type == "window") {
-    window_size <- params$window_size %||% 2L
-    for (i in seq_len(p - window_size + 1L)) {
-      for (j in seq(i, i + window_size - 2L)) {
-        from <- m[, j]
-        for (k in seq(j + 1L, i + window_size - 1L)) {
-          to <- m[, k]
-          any_na <- is.na(from) | is.na(to)
-          new_trans <- cbind(idx, from, to)[!any_na, , drop = FALSE]
-          trans[new_trans] <- trans[new_trans] + 1L
-        }
-      }
-    }
   } else if (type == "attention") {
     lambda <- params$lambda %||% 1.0
     decay <- params$decay %||% function(i, j, lambda) exp(-abs(i - j) / lambda)
@@ -743,7 +759,6 @@ compute_transitions <- function(m, a, type, params) {
             to <- m[, j]
             any_na <- is.na(from) | is.na(to)
             new_trans <- cbind(idx, from, to)[!any_na, , drop = FALSE]
-            #trans[new_trans] <- trans[new_trans] + exp((i - j) / lambda)
             d <- decay(time[, i], time[, j], lambda)[!any_na]
             trans[new_trans] <- trans[new_trans] + d
           }
@@ -756,7 +771,6 @@ compute_transitions <- function(m, a, type, params) {
             to <- m[, j]
             any_na <- is.na(from) | is.na(to)
             new_trans <- cbind(idx, from, to)[!any_na, , drop = FALSE]
-            #trans[new_trans] <- trans[new_trans] + exp((i - j) / lambda)
             d <- decay(time[, i], time[, j], lambda)[!any_na]
             trans[new_trans] <- trans[new_trans] + d
           }
@@ -767,6 +781,86 @@ compute_transitions <- function(m, a, type, params) {
   trans
 }
 
+compute_transitions_windowed <- function(m, a, type, params) {
+  n <- nrow(m)
+  p <- ncol(m)
+  idx <- seq_len(n)
+  trans <- array(0L, dim = c(n, a, a))
+  seq_lengths <- .rowSums(!is.na(m), m = n, n = p)
+  weight <- ifelse_(isTRUE(params$weighted), 1.0 / seq_lengths, rep(1L, n))
+  window_size <- params$window_size %||% 2L
+  window_span <- params$window_span %||% 1L
+  window_size <- window_size * window_span
+  divides <- p %% window_size == 0L
+  q <- p %/% window_size - 1L * divides
+  if (type %in% c("relative", "frequency")) {
+    for (i in seq_len(q)) {
+      j_idx <- seq(
+        (i - 1) * window_size + 1L,
+        i * window_size
+      )
+      for (j in j_idx) {
+        from <- m[, j]
+        k_idx <- seq(
+          i * window_size + 1L,
+          min(p, (i + 1) * window_size)
+        )
+        for (k in k_idx) {
+          to <- m[, k]
+          any_na <- is.na(from) | is.na(to)
+          new_trans <- cbind(idx, from, to)[!any_na, , drop = FALSE]
+          trans[new_trans] <- trans[new_trans] + 1L
+        }
+      }
+    }
+  } else if (type == "co-occurrence") {
+    for (i in seq_len(q + 1)) {
+      j_idx <- seq(
+        (i - 1) * window_size + 1L,
+        min(p, i * window_size)
+      )
+      for (j in j_idx) {
+        from <- m[, j]
+        k_idx <- seq(
+          (i - 1) * window_size + 1L,
+          min(p, i * window_size)
+        )
+        for (k in k_idx) {
+          to <- m[, k]
+          any_na <- is.na(from) | is.na(to)
+          new_trans <- cbind(idx, from, to)[!any_na, , drop = FALSE]
+          trans[new_trans] <- trans[new_trans] + 1L
+        }
+      }
+    }
+    # for (i in seq_len(q)) {
+    #   for (j in seq(i + 1, q + 1)) {
+    #     k_idx <- seq(
+    #       (i - 1) * window_size + 1L,
+    #       i * window_size
+    #     )
+    #     for (k in k_idx) {
+    #       from <- m[, k]
+    #       l_idx <- seq(
+    #         (j - 1) * window_size + 1L,
+    #         min(p, j * window_size)
+    #       )
+    #       for (l in l_idx) {
+    #         to <- m[, l]
+    #         any_na <- is.na(from) | is.na(to)
+    #         new_trans <- rbind(
+    #           cbind(idx, from, to)[!any_na, , drop = FALSE],
+    #           cbind(idx, to, from)[!any_na, , drop = FALSE]
+    #         )
+    #         trans[new_trans] <- trans[new_trans] + weight[!any_na]
+    #       }
+    #     }
+    #   }
+    # }
+  }
+  trans
+}
+
 # Internal function to get all transitions
 get_transitions <- function(x) { # nocov start
   m <- x$data
@@ -814,7 +908,7 @@ compute_weights <- function(transitions, type, scaling, a) {
 #' @param a An `integer`, the number of states.
 #' @noRd
 scale_weights <- function(weights, type, scaling, a) {
-  if (type == "relative") {
+  if (type == "relative" || "probability" %in% scaling) {
     rs <- .rowSums(weights, m = a, n = a)
     pos <- which(rs > 0)
     weights[pos, ] <- weights[pos, ] / rs[pos]
