@@ -18,6 +18,7 @@
 #' * `"none"`: No scaling is performed. The weights are used as is.
 #' * `"minmax"`: Performs min-max normalization, i.e., the minimum value is
 #'   subtracted and the differences are scaled by the range.
+#' * `"max"`: Max-normalization: the values are divided by the maximum value.
 #' * `"rank"`: Applies min-max normalization to the ranks of the weights
 #'   (computed with `ties.method = "average"`).
 #' * `"zscore"`: Computes the standard score, i.e. the mean weight is
@@ -32,6 +33,11 @@
 #' * `"quantile"`: Uses the empirical quantiles of the weights
 #'   via [stats::ecdf].
 #'
+#' @param measures A `character` vector indicating which centrality
+#'   measures should be computed. See [centralities()] for the
+#'   available measures. No measures are included by default.
+#' @param network A `logical` value indicating whether network metrics should
+#'   be included in the comparison. The default is `TRUE`.
 #' @param ... Ignored.
 #' @return A `tna_comparison` object, which is a `list` containing the
 #' following elements:
@@ -68,14 +74,16 @@ compare <- function(x, ...) {
 
 #' @export
 #' @rdname compare
-compare.tna <- function(x, y, scaling = "none", ...) {
-  compare_(x, y, scaling = scaling, ...)
+compare.tna <- function(x, y, scaling = "none", measures = character(0),
+                        network = TRUE, ...) {
+  compare_(x, y, scaling = scaling, measures = measures, network = network, ...)
 }
 
 #' @export
 #' @rdname compare
-compare.matrix <- function(x, y, scaling = "none", ...) {
-  compare_(x, y, scaling = scaling, ...)
+compare.matrix <- function(x, y, scaling = "none", measures = character(0),
+                           network = TRUE, ...) {
+  compare_(x, y, scaling = scaling, measures = measures, network = network, ...)
 }
 
 #' Compare Grouped TNA Models with Comprehensive Metrics
@@ -87,25 +95,33 @@ compare.matrix <- function(x, y, scaling = "none", ...) {
 #' `character` string.
 #' @param j An `integer` index or the name of the secondary cluster as a
 #' `character` string.
-#' @param scaling See [compare.tna()].
+#' @inheritParams compare
 #' @param ... Additional arguments passed to [compare.tna()].
 #' @return A `tna_comparison` object. See [compare.tna()] for details.
 #' @examples
 #' model <- group_model(engagement_mmm)
 #' compare(model, i = 1, j = 2)
 #'
-compare.group_tna <- function(x, i = 1L, j = 2L, scaling = "none", ...) {
+compare.group_tna <- function(x, i = 1L, j = 2L, scaling = "none",
+                              measures = character(0), network = TRUE, ...) {
   check_missing(x)
   check_class(x, "group_tna")
   check_clusters(x, i, j)
-  compare_(x = x[[i]], y = x[[j]], scaling = scaling, ...)
+  compare_(
+    x = x[[i]],
+    y = x[[j]],
+    scaling = scaling,
+    measures = measures,
+    network = network,
+    ...
+  )
 }
 
 #' Internal compare function
 #'
 #' @inheritParams compare
 #' @noRd
-compare_ <- function(x, y, scaling = "none", ...) {
+compare_ <- function(x, y, scaling = "none", measures, network, ...) {
   stopifnot_(
     is_tna(x) || is.matrix(x),
     "Argument {.arg x} must be a {.cls tna} object or a numeric {.cls matrix}."
@@ -114,16 +130,6 @@ compare_ <- function(x, y, scaling = "none", ...) {
     is_tna(y) || is.matrix(y),
     "Argument {.arg x} must be a {.cls tna} object or a numeric {.cls matrix}."
   )
-  metrics_x <- ifelse_(
-    is.matrix(x),
-    summary(build_model_(x)),
-    summary(x)
-  )
-  metrics_y <- ifelse_(
-    is.matrix(y),
-    summary(build_model_(y)),
-    summary(y)
-  )
   x <- ifelse_(is_tna(x), x$weights, x)
   y <- ifelse_(is_tna(y), y$weights, y)
   x_arg <- ifelse_(is_tna(x), "x$weights", "x")
@@ -149,6 +155,7 @@ compare_ <- function(x, y, scaling = "none", ...) {
   scaling_methods <- list(
     none = identity,
     minmax = ranger,
+    max = function(w) w / max(w, na.rm = TRUE),
     rank = function(w) ranger(rank(w, ties.method = "average")),
     zscore = function(w) (w - mean(w)) / stats::sd(w),
     log = log,
@@ -172,10 +179,8 @@ compare_ <- function(x, y, scaling = "none", ...) {
   edges <- expand.grid(source = rn, target = rn)
   edges_x <- edges
   edges_y <- edges
-  # edges_diff <- edges
   edges_x$weight <- as.vector(x)
   edges_y$weight <- as.vector(y)
-  # edges_diff$difference <- diff
   edges_combined <- edges
   edges_combined$weight_x <- edges_x$weight
   edges_combined$weight_y <- edges_y$weight
@@ -196,7 +201,7 @@ compare_ <- function(x, y, scaling = "none", ...) {
     edges_combined$standardized_weight_x / edges_combined$standardized_weight_y
 
   # Summary metrics
-  weight_dev <- data.frame(
+  weight_dev <- list(
     category = "Weight Deviations",
     metric = c(
       "Mean Abs. Diff.",
@@ -215,7 +220,7 @@ compare_ <- function(x, y, scaling = "none", ...) {
       stats::sd(x_vec) * mean(y_vec) / (mean(x_vec) * stats::sd(y_vec))
     )
   )
-  correlations <- data.frame(
+  correlations <- list(
     category = "Correlations",
     metric = c("Pearson", "Spearman", "Kendall", "Distance"),
     value = c(
@@ -225,7 +230,7 @@ compare_ <- function(x, y, scaling = "none", ...) {
       distance_correlation(x_vec, y_vec)
     )
   )
-  dissimilarities <- data.frame(
+  dissimilarities <- list(
     category = "Dissimilarities",
     metric = c(
       "Euclidean",
@@ -242,7 +247,7 @@ compare_ <- function(x, y, scaling = "none", ...) {
       sqrt(sum(abs_diff^2)) / sqrt(n / 2)
     )
   )
-  similarities <- data.frame(
+  similarities <- list(
     category = "Similarities",
     metric = c("Cosine", "Jaccard", "Dice", "Overlap", "RV"),
     value = c(
@@ -253,7 +258,7 @@ compare_ <- function(x, y, scaling = "none", ...) {
       rv_coefficient(x, y)
     )
   )
-  pattern_metrics <- data.frame(
+  pattern_metrics <- list(
     category = "Pattern Similarities",
     metric = c("Rank Agreement", "Sign Agreement"),
     value = c(
@@ -261,7 +266,7 @@ compare_ <- function(x, y, scaling = "none", ...) {
       mean(sign(x) == sign(y))
     )
   )
-  summary_metrics <- rbind(
+  summary_metrics <- dplyr::bind_rows(
     weight_dev,
     correlations,
     dissimilarities,
@@ -270,50 +275,71 @@ compare_ <- function(x, y, scaling = "none", ...) {
   )
 
   # Network metrics
-  network_metrics <- cbind(metrics_x, metrics_y[, -1L])
-  names(network_metrics) <- c("metric", "x", "y")
+  if (network) {
+    metrics_x <- ifelse_(
+      is.matrix(x),
+      summary(build_model_(x)),
+      summary(x)
+    )
+    metrics_y <- ifelse_(
+      is.matrix(y),
+      summary(build_model_(y)),
+      summary(y)
+    )
+    network_metrics <- cbind(metrics_x, metrics_y[, -1L])
+    names(network_metrics) <- c("metric", "x", "y")
+  }
 
   # Centralities
-  cents_x <- centralities(x) |>
-    tidyr::pivot_longer(
-      cols = !(!!rlang::sym("state")), names_to = "centrality", values_to = "x"
-    )
-  cents_y <- centralities(y) |>
-    tidyr::pivot_longer(
-      cols = !(!!rlang::sym("state")), names_to = "centrality", values_to = "y"
-    )
-  cents_xy <- cents_x
-  cents_xy$y <- cents_y$y
-  cents_xy$difference <- cents_xy$x - cents_xy$y
-  corr_fun <- function(x, y) {
-    out <- try(
-      stats::cor(x, y, use = "complete.obs"),
-      silent = TRUE
-    )
-    # Return NA in case of not insufficient pairs
-    ifelse_(
-      inherits(out, "try-error"),
-      NA_real_,
-      out
-    )
+  n_measures <- length(measures)
+  include_centralities <- n_measures > 0L
+  if (include_centralities) {
+    cents_x <- centralities(x, measures = measures) |>
+      tidyr::pivot_longer(
+        cols = !(!!rlang::sym("state")),
+        names_to = "centrality",
+        values_to = "x"
+      )
+    cents_y <- centralities(y, measures = measures) |>
+      tidyr::pivot_longer(
+        cols = !(!!rlang::sym("state")),
+        names_to = "centrality",
+        values_to = "y"
+      )
+    cents_xy <- cents_x
+    cents_xy$y <- cents_y$y
+    cents_xy$difference <- cents_xy$x - cents_xy$y
+    corr_fun <- function(x, y) {
+      out <- try_(stats::cor(x, y, use = "complete.obs"))
+      # Return NA in case of not insufficient pairs
+      ifelse_(
+        inherits(out, "try-error"),
+        NA_real_,
+        out
+      )
+    }
+    cents_corr <- cents_xy |>
+      dplyr::group_by(!!rlang::sym("centrality")) |>
+      dplyr::summarize(
+        Centrality = dplyr::first(!!rlang::sym("centrality")),
+        correlation = corr_fun(!!rlang::sym("x"), !!rlang::sym("y"))
+      )
+  }
+  out <- list(
+    matrices = list(x = x, y = y),
+    difference_matrix = d,
+    edge_metrics = edges_combined,
+    summary_metrics = summary_metrics
+  )
+  if (network) {
+    out$network_metrics <- network_metrics
+  }
+  if (include_centralities) {
+    out$centrality_differences = cents_xy
+    out$centrality_correlations = cents_corr
   }
-  cents_corr <- cents_xy |>
-    dplyr::group_by(!!rlang::sym("centrality")) |>
-    dplyr::summarize(
-      Centrality = dplyr::first(!!rlang::sym("centrality")),
-      correlation = corr_fun(!!rlang::sym("x"), !!rlang::sym("y"))
-    )
-
   structure(
-    list(
-      matrices = list(x = x, y = y),
-      difference_matrix = d,
-      edge_metrics = tibble::tibble(edges_combined),
-      summary_metrics = tibble::tibble(summary_metrics),
-      network_metrics = tibble::tibble(network_metrics),
-      centrality_differences = cents_xy,
-      centrality_correlations = cents_corr
-    ),
+    out,
     class = "tna_comparison"
   )
 }
@@ -365,10 +391,12 @@ rv_coefficient <- function(x, y) {
 #'
 #' Performs comprehensive sequence comparison analysis between groups. All
 #' patterns of the sequences (subsequences of specific length) are extracted
-#' from all sequences in each group. For two groups, the proportions of the
-#' patterns between the groups are compared with a z-test for proportions.
-#' For more than two groups, the Chi-squared  test is applied. Both tests are
-#' applied within patterns of the same length.
+#' from all sequences in each group. The pattern frequencies are compared
+#' between the groups using a permutation test. The reported effect size is the
+#' difference between the observed test statistic (sum of squared differences
+#' between the observed and expected counts) and the mean value over the
+#' permutation samples divided by their standard deviation times square root of
+#' the number of observations.
 #'
 #' @export
 #' @family comparison
@@ -379,25 +407,39 @@ rv_coefficient <- function(x, y) {
 #'   rows/sequences of `x`. Alternatively, a single `character` string giving
 #'   the column name of the data that defines the group when `x` is a wide
 #'   format `data.frame` or a `tna_data` object.
-#' @param sub An `integer` vector of subsequence lengths to analyze.
+#' @param sub An `integer` vector of pattern lengths to analyze.
 #'   The default is `2:5`.
 #' @param min_freq An `integer` giving the minimum number of times that a
-#'   specific pattern has to be observed to be included in the analysis.
-#'   The default is `5`.
-#' @param correction A `character` string naming the multiple comparison
+#'   specific pattern has to be observed in each group to be included in the
+#'   analysis. The default is `5`.
+#' @param test A `logical` value indicating whether to test the differences
+#'   of pattern counts between the groups using a permutation test.
+#'   The default is `FALSE`.
+#' @param iter An `integer` giving the number of iterations for the permutation
+#'   test. The default is `1000`.
+#' @param adjust A `character` string naming the multiple comparison
 #'   correction method (default: `"bonferroni"`). Supports all
 #'   [stats::p.adjust] methods: `"holm"`, `"hochberg"`, `"hommel"`,
-#'   `"bonferroni"`, `"BH"`, `"BY"`, `"fdr"`, `"none"`.
+#'   `"bonferroni"`, `"BH"`, `"BY"`, `"fdr"`, `"none"`. The adjustment
+#'   is carried out within sequences of the same length.
 #' @param ... Not used.
 #' @return A `tna_sequence_comparison` object, which is a `data.frame` with
-#'   columns giving the names of the pattern, pattern frequencies, pattern
-#'   proportions (within patterns of the same length), test statistic values
+#'   columns giving the names of the patterns, pattern frequencies, pattern
+#'   proportions (within patterns of the same length), effect sizes,
 #'   and p-values of the tests.
 #'
 #' @examples
 #' group <- c(rep("High", 1000), rep("Low", 1000))
 #' comp <- compare_sequences(group_regulation, group)
 #'
+#' # With permutation test (small number of iterations for CRAN)
+#' comp_test <- compare_sequences(
+#'   group_regulation,
+#'   group,
+#'   test = TRUE,
+#'   iter = 10
+#' )
+#'
 #' @export
 compare_sequences <- function(x, ...) {
   UseMethod("compare_sequences")
@@ -406,14 +448,17 @@ compare_sequences <- function(x, ...) {
 #' @export
 #' @rdname compare_sequences
 compare_sequences.default <- function(x, group, sub, min_freq = 5L,
-                                      correction = "bonferroni", ...) {
+                                      test = FALSE, iter = 1000L,
+                                      adjust = "bonferroni", ...) {
   check_missing(group)
   model <- group_tna(x, group = group)
   compare_sequences.group_tna(
     x = model,
     sub = sub,
     min_freq = min_freq,
-    correction = correction,
+    test = test,
+    iter = iter,
+    adjust = adjust,
     ...
   )
 }
@@ -421,126 +466,138 @@ compare_sequences.default <- function(x, group, sub, min_freq = 5L,
 #' @export
 #' @rdname compare_sequences
 compare_sequences.group_tna <- function(x, sub, min_freq = 5L,
-                                        correction = "bonferroni", ...) {
+                                        test = FALSE, iter = 1000L,
+                                        adjust = "bonferroni", ...) {
   check_missing(x)
   check_class(x, "group_tna")
-  sub <- sub %m% seq(2, min(5, ncol(x[[1L]]$data)))
+  sub <- sub %m% seq(1, min(5, ncol(x[[1L]]$data)))
   check_range(
     sub,
     type = "integer",
     scalar = FALSE,
-    lower = 2L,
+    lower = 1L,
     upper = ncol(x[[1L]]$data)
   )
   check_values(min_freq)
-  correction <- check_match(correction, stats::p.adjust.methods)
-  n_group <- length(x)
-  groups <- names(x)
-  patterns <- vector(mode = "list", length = n_group)
-  names(patterns) <- groups
-  for (g in groups) {
-    m <- matrix(x[[g]]$labels[x[[g]]$data], nrow = nrow(x[[g]]$data))
-    patterns[[g]] <- extract_subsequences(m = m, q = sub)
-  }
+  check_flag(test)
+  check_values(iter, strict = TRUE)
+  adjust <- check_match(adjust, stats::p.adjust.methods)
   compare_sequences_(
-    x = patterns,
-    q = sub,
+    x = x,
+    len = sub,
     min_freq = min_freq,
-    correction = correction
+    test = test,
+    iter = iter,
+    adjust = adjust
   )
 }
 
 # Internal sequence comparison
-compare_sequences_ <- function(x, q, min_freq, correction) {
-  groups <- names(x)
-  n_group <- length(groups)
-  p <- length(q)
-  patterns <- vector(mode = "list", length = p)
-  residuals <- vector(mode = "list", length = p)
-  for (j in q) {
-    pat <- lapply(x, "[[", j)
-    pat_u <- unique(unlist(pat))
-    pat <- data.frame(pattern = pat_u)
-    total <- numeric(n_group)
-    names(total) <- groups
-    for (g in groups) {
-      freq_col <- paste0("freq_", g)
-      prop_col <- paste0("prop_", g)
-      freq_tab <- table(x[[g]][[j]])
-      pat[[freq_col]] <- as.numeric(freq_tab[pat_u])
-      pat[[freq_col]][is.na(pat[[freq_col]])] <- 0
-      total[g] <- sum(pat[[freq_col]])
-      pat[[prop_col]] <- ifelse_(
-        total[g] > 0,
-        pat[[freq_col]] / total[g],
-        0.0
-      )
-    }
-    n <- nrow(pat)
-    freq_cols <- paste0("freq_", groups)
-    valid_patterns <- n_group == .rowSums(
-      pat[, freq_cols] >= min_freq,
-      m = n,
-      n = n_group
+compare_sequences_ <- function(x, len, min_freq, test, iter, adjust) {
+  data <- combine_data(x)
+  group <- data$.group
+  data$.group <- NULL
+  groups <- unique(group)
+  m <- as.matrix(data)
+  n <- nrow(m)
+  k <- length(len)
+  labels <- x[[1L]]$labels
+  pattern_matrices <- extract_patterns(m, len, labels)
+  patterns <- factorize_patterns(pattern_matrices, group)
+  perm <- onlyif(
+    test,
+    permutation_test_patterns(x, len, iter, adjust)
+  )
+  out <- vector(mode = "list", length = k)
+  freq_cols <- paste0("freq_", groups)
+  for (j in seq_len(k)) {
+    pat <- patterns[[j]]
+    cs <- colSums(pat)
+    tmp <- cbind(pat, pat / cs[col(pat)])
+    tmp[!is.finite(tmp)] <- NA_real_
+    colnames(tmp) <- c(freq_cols, paste0("prop_", groups))
+    out[[j]] <- cbind(
+      data.frame(pattern = pattern_matrices[[j]]$unique),
+      as.data.frame(tmp)
     )
-    pat <- pat[valid_patterns, ]
-    n <- nrow(pat)
-    if (n > 0L) {
-      freq_mat <- as.matrix(pat[, freq_cols])
-      resid <- matrix(0.0, nrow = n, ncol = n_group)
-      rownames(resid) <- pat$pattern
-      colnames(resid) <- groups
-      pat$statistic <- NA
-      pat$p_value <- NA
-      for (i in seq_len(n)) {
-        tab <- matrix(c(freq_mat[i, ], total), nrow = 2L, byrow = TRUE)
-        chisq <- suppressWarnings(stats::chisq.test(tab))
-        pat$statistic[i] <- chisq$statistic
-        pat$p_value[i] <- stats::p.adjust(chisq$p.value, method = correction)
-        resid[i, ] <- chisq$stdres[1L, ]
-      }
-      patterns[[j]] <- pat
-      residuals[[j]] <- resid
-    }
   }
-  valid_patterns <- vapply(patterns, function(y) !is.null(y), logical(1L))
+  out <- do.call(base::rbind, out)
+  if (test) {
+    out <- cbind(out, perm) |>
+      dplyr::arrange(!!rlang::sym("p_value"))
+  }
+  rmin <- apply(out[, freq_cols], 1, min)
+  valid_patterns <- rmin >= min_freq
   stopifnot_(
     any(valid_patterns),
-    "No common patterns with frequency greater than {min_freq} were found."
+    "No common patterns with a frequency greater than {min_freq} were found."
   )
+  out <- out[valid_patterns, ]
   structure(
-    dplyr::bind_rows(patterns) |>
-      dplyr::arrange(!!rlang::sym("p_value")),
-    residuals = do.call("rbind", residuals),
+    out,
+    groups = groups,
+    freq_cols = freq_cols,
     class = c("tna_sequence_comparison", "data.frame")
   )
 }
 
-extract_subsequences <- function(m, q) {
+extract_patterns <- function(m, len, labels) {
   n <- nrow(m)
   p <- ncol(m)
+  k <- length(len)
   mis <- is.na(m)
-  out <- vector(mode = "list", length = max(q))
-  for (j in q) {
-    tmp <- character(n * (p - j + 1L))
-    idx <- seq(1L, n)
-    for (i in seq_len(p - 1L)) {
-      if (i + j - 1L > p) {
-        next
+  out <- vector(mode = "list", length = k)
+  for (l in seq_len(k)) {
+    j <- len[l]
+    tmp <- matrix("", n,  p - j + 1L)
+    for (i in seq_len(p - j + 1L)) {
+      idx <- i:(i + j - 1L)
+      pattern <- labels[m[, idx]]
+      dim(pattern) <- c(n, j)
+      pattern_mis <- mis[, idx]
+      valid <- .rowSums(pattern_mis, m = n, n = j) == 0L
+      if (any(valid)) {
+        subseq <- do.call(
+          base::paste,
+          c(
+            as.data.frame(
+              pattern[valid, , drop = FALSE],
+              stringsAsFactors = FALSE
+            ),
+            sep = "->"
+          )
+        )
+        tmp[valid, i] <- subseq
       }
-      pattern_idx <- i:(i + j - 1L)
-      pattern <- m[, pattern_idx]
-      pattern_mis <- mis[, pattern_idx]
-      invalid <- .rowSums(pattern_mis, m = n, n = j) > 0
-      subseq <- do.call(
-        paste,
-        c(as.data.frame(pattern, stringsAsFactors = FALSE), sep = "-")
-      )
-      subseq[invalid] <- ""
-      tmp[idx] <- subseq
-      idx <- idx + n
     }
-    out[[j]] <- tmp[nzchar(tmp)]
+    out[[l]] <- list(
+      patterns = t(tmp),
+      unique = sort(unique(tmp[nzchar(tmp)]))
+    )
+  }
+  out
+}
+
+factorize_patterns <- function(patterns, group) {
+  groups <- unique(group)
+  p <- length(patterns)
+  g <- length(groups)
+  out <- vector(mode = "list", length = p)
+  group_idx <- vector(mode = "list", length = p)
+  for (j in seq_len(g)) {
+    group_idx[[j]] <- which(group == groups[j])
+  }
+  for (i in seq_len(p)) {
+    pat <- patterns[[i]]$pat
+    pat_u <- patterns[[i]]$unique
+    n <- length(pat_u)
+    tmp <- matrix(0L, nrow = n, ncol = g)
+    for (j in seq_len(g)) {
+      y <- pat[, group_idx[[j]]]
+      idx <- match(y, pat_u, nomatch = 0)
+      tmp[, j] <- tabulate(idx, nbins = n)
+    }
+    out[[i]] <- tmp
   }
   out
 }
