@@ -189,6 +189,19 @@ test_that("comparison results can be printed", {
   )
 })
 
+test_that("comparison results with centralities can be printed", {
+  model_x <- tna(group_regulation[1:200, ])
+  model_y <- tna(group_regulation[1001:1200, ])
+  # Comparing models with centrality measures included
+  comp <- compare(model_x, model_y, measures = c("InStrength", "OutStrength"))
+  expect_true(!is.null(comp$centrality_differences))
+  expect_true(!is.null(comp$centrality_correlations))
+  expect_error(
+    capture.output(print(comp)),
+    NA
+  )
+})
+
 test_that("permutation test results can be printed for clusters", {
   perm <- permutation_test(mmm_model, iter = 50)
   expect_error(
@@ -203,7 +216,7 @@ test_that("permutation test results can be printed for clusters", {
 #     NA
 #   )
 # })
-
+# 
 # test_that("mixture Markov model summary can be printed", {
 #   expect_error(
 #     capture.output(print(summary(engagement_tna_mmm))),
@@ -226,4 +239,12 @@ test_that("sequence comparison output can be printed", {
     capture.output(print(comp)),
     NA
   )
-})
\ No newline at end of file
+})
+
+test_that("reliability analysis can be printed", {
+  rel <- reliability(mock_tna_seq, iter = 50)
+  expect_error(
+    capture.output(print(rel)),
+    NA
+  )
+})
